# Mobicoop V3 - User Service

User-related data management.

## Requirements

You need [Docker](https://docs.docker.com/engine/) and [Docker-compose](https://docs.docker.com/compose/).

A RabbitMQ instance is also required to send / receive messages when data has been inserted/updated/deleted.

## Installation

Copy `.env.dist` to `.env` :

```bash
cp .env.dist .env
```

and modify it to suit your needs.

Then execute :

```bash
docker-compose up -d
```

The app runs automatically on the port defined in `SERVICE_PORT` of `.env` file (default : _5001_).

You then need to set the appropriate rights for PGAdmin container :

```bash
sudo chown -R 5050:5050 postgresql/.pgadmin_data
```

## Database migration

Before using the app, you need to launch the database migration :

```bash
docker exec v3-user sh -c "npx prisma migrate dev"
```

## Usage

The app exposes the following [gRPC](https://grpc.io/) services :

-   **FindByUuid** : find a user by its uuid

    ```json
    {
        "uuid": "80126a61-d128-4f96-afdb-92e33c75a3e1"
    }
    ```

-   **FindAll** : find all users; you can use pagination with `page` (default:_1_) and `perPage` (default:_10_)

    ```json
    {
        "page": 1,
        "perPage": 10
    }
    ```

-   **Create** : create a user (note that uuid is optional, a uuid will be automatically attributed if it is not provided)

    ```json
    {
        "email": "jezabel.doe@email.com",
        "firstName": "Jezabel",
        "lastName": "Doe",
        "phone": "+33611223344"
    }
    ```

-   **Update** : update a user

    ```json
    {
        "firstName": "Jezabel-Katarina",
        "email": "jezabel-katarina.doe@email.com",
        "uuid": "30f49838-3f24-42bb-a489-8ffb480173ae"
    }
    ```

-   **Delete** : delete a user by its uuid

    ```json
    {
        "uuid": "80126a61-d128-4f96-afdb-92e33c75a3e1"
    }
    ```

## Messages

A mentionned earlier, RabbitMQ messages are sent after these events :

-   **Create** (message : the created user informations)

-   **Update** (message : the updated user informations)

-   **Delete** (message : the uuid of the deleted user)

Various messages are also sent for logging purpose.

## Test

```bash
# unit tests
docker exec v3-user sh -c "npm run test"

# test coverage
docker exec v3-user sh -c "npm run test:cov"
```

Note : you can run all npm commands directly _outside_ the container (see _scripts_ section of `package.json` for available commands), but you need NodeJS installed locally. We **strongly** advise to install [Node Version Manager](https://github.com/nvm-sh/nvm) and use the latest LTS version of Node.

## License

Mobicoop V3 - User Service is [AGPL licensed](LICENSE).
